# Generated by Django 3.2.8 on 2022-05-19 15:28

from django.db import migrations, models

import ai_django_core.mixins.validation


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0004_auto_20210511_1343'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelWithCleanMixin',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            bases=(ai_django_core.mixins.validation.CleanOnSaveMixin, models.Model),
        ),
    ]



# Generated by Django 3.1 on 2021-04-19 14:16

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('testapp', '0002_auto_210407'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelWithFkToSelf',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'parent',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='children',
                        to='testapp.modelwithfktoself',
                    ),
                ),
            ],
        ),
        migrations.AlterField(
            model_name='foreignkeyrelatedmodel',
            name='single_signal',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='foreign_key_related_models',
                to='testapp.mysinglesignalmodel',
            ),
        ),
        migrations.AlterField(
            model_name='modelwithfktoself',
            name='parent',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='children',
                to='testapp.modelwithfktoself',
            ),
        ),
        migrations.CreateModel(
            name='ModelWithOneToOneToSelf',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'peer',
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='related_peer',
                        to='testapp.modelwithonetoonetoself',
                    ),
                ),
            ],
        ),
    ]



from pathlib import Path

BASE_PATH = Path(__file__).resolve(strict=True).parent

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'testapp',
)

DEBUG = False

ALLOWED_HOSTS = ['localhost:8000']

SECRET_KEY = 'ASDFjklö123456890'

# Routing
ROOT_URLCONF = 'testapp.urls'

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite',
    }
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': ['templates'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
            'debug': True,
        },
    },
]

MIDDLEWARE = (
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

# Mail backend
EMAIL_BACKEND_DOMAIN_WHITELIST = ''
EMAIL_BACKEND_REDIRECT_ADDRESS = ''

TIME_ZONE = 'UTC'

LOCALE_PATHS = [str(BASE_PATH) + '/ai_django_core/locale']



from django.contrib import admin
from django.contrib.auth.models import User
from django.http import HttpRequest
from django.test import TestCase

from ai_django_core.admin.model_admins.mixins import AdminRequestInFormMixin
from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import MySingleSignalModel


class TestAdminRequestInFormMixinAdmin(AdminRequestInFormMixin, admin.ModelAdmin):
    pass


class AdminRequestInFormMixinTest(RequestProviderMixin, TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.super_user = User.objects.create(username='super_user', is_superuser=True)

        admin.site.register(MySingleSignalModel, TestAdminRequestInFormMixinAdmin)

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

        admin.site.unregister(MySingleSignalModel)

    def test_request_is_in_form(self):
        model_admin = TestAdminRequestInFormMixinAdmin(model=MySingleSignalModel, admin_site=admin.site)
        form = model_admin.get_form(self.get_request(self.super_user))

        self.assertIsInstance(form.request, HttpRequest)



from django.contrib import messages
from django.contrib.auth.models import AnonymousUser, User
from django.contrib.sessions.backends.base import SessionBase
from django.http import HttpRequest
from django.test import TestCase

from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import MySingleSignalModel


class RequestProviderMixinTest(RequestProviderMixin, TestCase):
    def test_request_is_request(self):
        request = self.get_request(None)
        self.assertIsInstance(request, HttpRequest)

    def test_request_user_set(self):
        user = User.objects.create(username='albertus_magnus')
        request = self.get_request(user)
        self.assertEqual(request.user, user)

    def test_request_user_is_none_working(self):
        request = self.get_request(None)
        self.assertEqual(request.user, None)

    def test_django_messages_set_up_correctly(self):
        request = self.get_request(None)

        # This would fail if the django messages were not set up correctly
        messages.add_message(request, messages.SUCCESS, 'I am a great message!')

        self.assertIsInstance(request.session, SessionBase)

    def test_django_session_set_up_correctly(self):
        request = self.get_request(None)
        request.session['my_val'] = 27
        request.session.modified = True

        self.assertEqual(request.session['my_val'], 27)

    def test_passed_user_is_none(self):
        request = self.get_request(None)
        self.assertIsNone(request.user)

    def test_passed_user_is_regular_user(self):
        user = User.objects.create(username='albertus_magnus')
        request = self.get_request(user)
        self.assertEqual(request.user, user)

    def test_passed_user_is_anonymous_user(self):
        anonymous_user = AnonymousUser()
        request = self.get_request(anonymous_user)
        self.assertEqual(request.user, anonymous_user)

    def test_passed_user_is_other_type(self):
        wrong_object = MySingleSignalModel()
        with self.assertRaises(ValueError):
            self.get_request(wrong_object)

    def test_default_url_used(self):
        request = self.get_request()
        self.assertEqual(request.build_absolute_uri(), 'http://testserver/')

    def test_passed_url_used(self):
        request = self.get_request(url='my-url')
        self.assertEqual(request.build_absolute_uri(), 'http://testserver/my-url')



# Generated by Django 3.1 on 2021-05-11 13:43

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('testapp', '0003_modelwithfktoself'),
    ]

    operations = [
        migrations.AlterField(
            model_name='commoninfobasedmodel',
            name='created_at',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Created at'),
        ),
        migrations.AlterField(
            model_name='commoninfobasedmodel',
            name='created_by',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name='testapp_commoninfobasedmodel_created',
                to=settings.AUTH_USER_MODEL,
                verbose_name='Created by',
            ),
        ),
        migrations.AlterField(
            model_name='commoninfobasedmodel',
            name='lastmodified_at',
            field=models.DateTimeField(
                db_index=True, default=django.utils.timezone.now, verbose_name='Last modified at'
            ),
        ),
        migrations.AlterField(
            model_name='commoninfobasedmodel',
            name='lastmodified_by',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name='testapp_commoninfobasedmodel_lastmodified',
                to=settings.AUTH_USER_MODEL,
                verbose_name='Last modified by',
            ),
        ),
    ]



import datetime
import os
from typing import Optional, Union

from django.contrib.humanize.templatetags.humanize import intcomma
from django.template.defaultfilters import floatformat, slugify
from django.utils.encoding import smart_str

from ai_django_core.utils import datetime_format


def distinct(not_distinct_list: list) -> list:
    """
    Returns a list with no duplicate elements
    """
    return list(set(not_distinct_list))


def slugify_file_name(file_name: str, length: int = 40) -> str:
    """
    Slugify the given file name
    """
    name, ext = os.path.splitext(file_name)
    name = smart_str(slugify(name).replace('-', '_'))
    ext = smart_str(slugify(ext))
    result = '{}{}{}'.format(name[:length], "." if ext else "", ext)
    return result


def smart_truncate(text: str, max_length: int = 100, suffix: str = '...') -> str:
    """
    Returns a string of at most `max_length` characters, cutting
    only at word-boundaries. If the string was truncated, `suffix`
    will be appended.
    In comparison to Djangos default filter `truncatechars` this method does NOT break words and you
    can choose a custom suffix.
    """
    if text is None:
        return ''

    # Return the string itself if length is smaller or equal to the limit
    if len(text) <= max_length:
        return text

    # Cut the string
    value = text[:max_length]

    # Break into words and remove the last
    words = value.split(' ')[:-1]

    # Join the words and return
    return ' '.join(words) + suffix


def float_to_string(value: Optional[float], replacement: str = "0,00") -> str:
    """
    Converts a float to a properly, German-formatted string value
    # todo name is misleading, should contain "de"
    # todo thousand separator would be nice
    If the passed object is None, it will return `replacement`.
    """
    return ("%.2f" % value).replace('.', ',') if value is not None else replacement


def date_to_string(value: Optional[datetime.date], replacement: str = "-", str_format: str = "%d.%m.%Y") -> str:
    """
    Converts a given date "value" to a string formatted with `str_format`.
    If the passed "value" is None, it will return `replacement`.
    """
    return value.strftime(str_format) if value is not None else replacement


def datetime_to_string(
    value: Optional[datetime.datetime], replacement: str = "-", str_format: str = "%d.%m.%Y %H:%M"
) -> str:
    """
    Converts a given datetime "value" to a string formatted with `str_format`.
    If the passed "value" is None, it will return `replacement`.
    """
    return datetime_format(value, str_format) if value is not None else replacement


def number_to_string(value: Optional[Union[int, float]], decimal_digits: int = 0, replacement: str = "-") -> str:
    """
    Converts a given int or float number to a string. Decimal places can be configured via `decimal_digits`.
    If the passed "value" is None, it will return `replacement`.
    Attention: Will not localise the return value!
    """
    return intcomma(floatformat(value, decimal_digits)) if value is not None else replacement


def string_or_none_to_string(value: Optional[any], replacement: str = "-") -> str:
    """
    Converts a given "value" to a string.
    If the passed "value" is None, it will return `replacement`.
    """
    return value if value is not None else replacement


def encode_to_xml(text: str) -> str:
    """
    Encodes ampersand, greater and lower characters in a given string to HTML-entities.
    """
    text_str = str(text)
    text_str = text_str.replace('&', '&amp;')
    text_str = text_str.replace('<', '&lt;')
    text_str = text_str.replace('>', '&gt;')

    return text_str



from unittest import mock

from django.contrib import admin
from django.contrib.auth.models import User
from django.test import TestCase

from ai_django_core.admin.model_admins.mixins import FetchObjectMixin
from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import MySingleSignalModel


class TestFetchObjectMixinAdmin(FetchObjectMixin, admin.ModelAdmin):
    pass


class MockResolverResponse:
    kwargs = None


class FetchObjectMixinTest(RequestProviderMixin, TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.super_user = User.objects.create(username='super_user', is_superuser=True)

        admin.site.register(MySingleSignalModel, TestFetchObjectMixinAdmin)

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

        admin.site.unregister(MySingleSignalModel)

    def test_model_is_set(self):
        obj = MySingleSignalModel.objects.create(value=1)
        model_admin = TestFetchObjectMixinAdmin(model=MySingleSignalModel, admin_site=admin.site)

        request = self.get_request(self.super_user)

        return_obj = MockResolverResponse()
        return_obj.kwargs = {'object_id': obj.id}
        with mock.patch('ai_django_core.admin.model_admins.mixins.resolve', return_value=return_obj):
            obj_from_request = model_admin.get_object_from_request(request)

        self.assertEqual(obj_from_request, obj)



from django.shortcuts import render
from django.views import generic
from django.views.defaults import ERROR_403_TEMPLATE_NAME
from django.views.generic.detail import SingleObjectMixin


class CustomPermissionMixin(generic.View):
    """
    This mixin provides the method `validate_permissions()` to create a space where custom, non-django-permissions
    can live. This method will be called in the `dispatch()` method to avoid executing unnecessary logic in the
    "permission denied" case.
    """

    def validate_permissions(self) -> bool:
        return True

    def dispatch(self, request, *args, **kwargs):
        if self.validate_permissions():
            return super().dispatch(request, *args, **kwargs)
        else:
            return render(self.request, ERROR_403_TEMPLATE_NAME, status=403)


class RequestInFormKwargsMixin:
    """
    Injects the request in the form.
    Attention: Have to be removed in the init of the form (via .pop())
    """

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update({'request': self.request})
        return kwargs


class ToggleView(SingleObjectMixin, generic.View):
    """
    Generic view for updating an object without any user data being sent. Therefore, we don't need a form to validate
    user input.
    Most common use-case is toggling a flag inside an object.
    """

    http_method_names = ('post',)

    def post(self, request, *args, **kwargs):
        raise NotImplementedError



from django.contrib import admin
from django.contrib.auth.models import User
from django.test import TestCase

from ai_django_core.admin.model_admins.inlines import ReadOnlyTabularInline
from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import ForeignKeyRelatedModel, MySingleSignalModel


class TestReadOnlyTabularInline(ReadOnlyTabularInline):
    model = ForeignKeyRelatedModel


class AdminInlineTest(RequestProviderMixin, TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.super_user = User.objects.create(username='super_user', is_superuser=True)

    def test_read_only_tabular_inline_admin_all_fields_readonly(self):
        obj = MySingleSignalModel(value=1)
        fk_related_obj = ForeignKeyRelatedModel(single_signal=obj)

        admin_class = TestReadOnlyTabularInline(parent_model=MySingleSignalModel, admin_site=admin.site)
        readonly_fields = admin_class.get_readonly_fields(request=self.get_request(), obj=fk_related_obj)

        self.assertEqual(len(readonly_fields), 1)
        self.assertIn('single_signal', readonly_fields)

    def test_read_only_admin_tabular_inline_no_change_permissions(self):
        admin_class = TestReadOnlyTabularInline(parent_model=MySingleSignalModel, admin_site=admin.site)

        request = self.get_request(self.super_user)

        self.assertFalse(admin_class.has_add_permission(request))
        self.assertFalse(admin_class.has_change_permission(request))
        self.assertFalse(admin_class.has_delete_permission(request))



# Generated by Django 2.2.7 on 2020-10-29 17:08

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='MyMultipleSignalModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.PositiveIntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name='MySingleSignalModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.PositiveIntegerField(default=0)),
            ],
        ),
    ]



import sentry_sdk
from graphene_django.views import GraphQLView


class SentryGraphQLView(GraphQLView):
    """
    Graphene sentries are all merged into a single issue.
    This code wraps the normal GraphQLView with a handler that deals with Sentry errors properly.

    Copied from:
    https://github.com/phalt/graphene-django-sentry/blob/master/graphene_django_sentry/views.py

    This class was tested with the following versions of these libraries. You must have them installed. If your
    version does not meet the requirements this code may break. Use on your own risk.
        * sentry_sdk >= 0.13.0
        * graphene_django >=2.9.1, <3.0
    """

    def execute_graphql_request(self, *args, **kwargs):
        """Extract any exceptions and send them to Sentry"""
        result = super().execute_graphql_request(*args, **kwargs)

        if result.errors:
            self._capture_sentry_exceptions(result.errors)
        return result

    def _capture_sentry_exceptions(self, errors):
        """
        Capture each exception and get its original error to send it to sentry.
        """
        for error in errors:
            try:
                sentry_sdk.capture_exception(error.original_error)
            except AttributeError:
                sentry_sdk.capture_exception(error)



from typing import Union
from unittest import mock

from django.contrib.auth.models import AnonymousUser, Permission, User
from django.utils.translation import gettext_lazy as _

from ai_django_core.tests.errors import TestSetupConfigurationError
from ai_django_core.tests.mixins import RequestProviderMixin
from ai_django_core.view_layer.mixins import DjangoPermissionRequiredMixin


class BaseViewPermissionTestMixin(RequestProviderMixin):
    view_class = None
    permission_list = None
    view_kwargs = {}

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.user = cls.get_test_user()

    @classmethod
    def get_test_user(cls):
        return User.objects.create(username='test_user', email='test.user@ai-django-core.com')

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        if not self.view_class:
            raise TestSetupConfigurationError(_('BaseViewPermissionTestMixin used without setting a "view_class".'))

    def get_view_instance(self, *, user: Union[User, AnonymousUser], kwargs: dict = None, method: str = 'GET'):
        """
        Creates an instance of the given view class and injects a valid request.
        """
        request = self.get_request(user=user, method=method)
        view = self.view_class()
        view.kwargs = kwargs if kwargs else self.view_kwargs
        view.request = request
        return view

    def test_view_class_inherits_mixin(self):
        self.assertTrue(issubclass(self.view_class, DjangoPermissionRequiredMixin))

    def test_permissions_are_equal(self):
        # Sanity checks
        self.assertIsNotNone(self.permission_list, msg='Missing permission list declaration in test')
        self.assertIsNotNone(self.view_class.permission_list, msg='Missing permission list declaration in view')

        # Assert same amount of permissions
        self.assertEqual(len(self.permission_list), len(self.view_class.permission_list))

        # Compare target permissions to real permissions
        for permission in self.permission_list:
            self.assertIn(permission, list(self.view_class.permission_list))

        # Compare real permissions to target permissions
        for permission in self.view_class.permission_list:
            self.assertIn(permission, list(self.permission_list))

    def test_permissions_exist_in_database(self):
        for permission in self.permission_list:
            if '.' not in permission:
                raise TestSetupConfigurationError(
                    f'View "{self.view_class}" contains ill-formatted permission ' f'"{permission}".'
                )
            app_label, codename = permission.split('.')
            permission_qs = Permission.objects.filter(content_type__app_label=app_label, codename=codename)

            if not permission_qs.exists():
                raise TestSetupConfigurationError(
                    f'View "{self.view_class}" contains invalid permission ' f'"{permission}".'
                )

    def test_passes_login_barrier_is_called(self):
        with mock.patch.object(self.view_class, 'passes_login_barrier', return_value=False) as mock_method:
            view = self.get_view_instance(user=AnonymousUser())
            response = view.dispatch(request=view.request, **view.kwargs)
            # If a user is not logged in, he'll be forwarded to the login view
            self.assertEqual(response.status_code, 302)

        mock_method.assert_called_once()

    def test_has_permissions_is_called(self):
        with mock.patch.object(self.view_class, 'has_permissions', return_value=False) as mock_method:
            view = self.get_view_instance(user=self.user)
            response = view.dispatch(request=view.request, **view.kwargs)
            self.assertEqual(response.status_code, 403)

        mock_method.assert_called_once()



class CountChildrenFormsetMixin:
    """
    Provides a method to count the valid children of a formset. Takes care about records which are to be deleted.
    """

    def get_number_of_children(self):
        # Count all choices which are not being deleted right now
        no_choices = 0
        for form in self.forms:
            if getattr(form, 'cleaned_data', None) and not form.cleaned_data.get('DELETE'):
                no_choices += 1
        return no_choices



from django.contrib.auth.models import User
from django.test import TestCase

from testapp.models import MySingleSignalModel


class GloballyVisibleQuerySetTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        # Create test user
        cls.user = User.objects.create(username='my-username')

        # Create list of objects
        cls.object_list = [
            MySingleSignalModel.objects.create(value=1),
            MySingleSignalModel.objects.create(value=2),
        ]

    def test_visible_for_regular(self):
        self.assertGreater(len(self.object_list), 0)
        self.assertEqual(MySingleSignalModel.objects.visible_for(self.user).count(), len(self.object_list))

    def test_editable_for_regular(self):
        self.assertGreater(len(self.object_list), 0)
        self.assertEqual(MySingleSignalModel.objects.editable_for(self.user).count(), len(self.object_list))

    def test_deletable_for_regular(self):
        self.assertGreater(len(self.object_list), 0)
        self.assertEqual(MySingleSignalModel.objects.deletable_for(self.user).count(), len(self.object_list))



from typing import Optional, Union

from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.messages.storage.fallback import FallbackStorage
from django.contrib.sessions.middleware import SessionMiddleware
from django.http import HttpResponse
from django.test import RequestFactory
from django.utils.translation import gettext_lazy as _


class ClassBasedViewTestMixin:
    """
    Helper mixin for easy testing of Django views
    """

    factory_class = RequestFactory
    view_class = None

    @staticmethod
    def _authentication(request, user):
        request.user = user if user else AnonymousUser()

    def _get_response(self, method, user, data, url_params=None, *args, **kwargs):
        """Returns response."""

        # Catch case that URL does not get any params passed to
        if not url_params:
            url_params = {}

        # Create request
        factory = self.factory_class()
        req_kwargs = {}
        if data:
            req_kwargs.update({'data': data})
        request = getattr(factory, method)('/', **req_kwargs)

        # Annotate a request object with a session
        middleware = SessionMiddleware(get_response=HttpResponse(status=200))
        middleware.process_request(request)
        request.session.save()

        # Authenticate user
        self._authentication(request, user)

        # Setup messages
        messages = FallbackStorage(request)
        request._messages = messages

        # Call view
        return self.view_class.as_view()(request, *args, **url_params)

    def get(self, user=None, data=None, url_params=None, *args, **kwargs):
        """Returns response for a GET request."""
        return self._get_response('get', user, data, url_params, *args, **kwargs)

    def post(self, user=None, data=None, url_params=None, *args, **kwargs):
        """Returns response for a POST request."""
        return self._get_response('post', user, data, url_params, *args, **kwargs)

    def delete(self, user=None, data=None, url_params=None, *args, **kwargs):
        """Returns response for a DELETE request."""
        return self._get_response('delete', user, data, url_params, *args, **kwargs)


class RequestProviderMixin:
    """
    Mixin that provides a method which returns a django request.
    Supposed to be used in unittests that require a request object.
    Also adds the message middleware so django messages can be tested.
    """

    @staticmethod
    def get_request(
        user: Union[AbstractBaseUser, AnonymousUser, None] = None, method: str = 'GET', url: Optional[str] = None
    ):
        """
        Creates and returns a django request.
        """
        # Determine URL
        url = url if url else '/'

        # Create test request
        factory = RequestFactory()
        request = factory.get(url)

        # Set user object if it is of a valid type
        if user is None or isinstance(user, AbstractBaseUser) or isinstance(user, AnonymousUser):
            request.user = user
        else:
            raise ValueError(_('Please pass a user object to RequestProviderMixin.'))

        # Annotate a request object with a session
        session_middleware = SessionMiddleware(get_response=HttpResponse(status=200))
        session_middleware.process_request(request)
        request.session.save()

        # Annotate a request object with messages
        message_middleware = MessageMiddleware(get_response=HttpResponse(status=200))
        message_middleware.process_request(request)
        request.session.save()

        # Set request method
        request.method = method

        return request



class CleanOnSaveMixin:
    """
    Mixin which ensures model-level validation ("clean()") is called on saving the current instance.
    """

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)



from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe

register = template.Library()


def obfuscate_string(value):
    return ''.join([f'&#{str(ord(char)):s};' for char in value])


@register.filter
@stringfilter
def obfuscate(value):
    return mark_safe(obfuscate_string(value))


@register.filter
@stringfilter
def obfuscate_mailto(value, text=False):
    mail = obfuscate_string(value)

    if text:
        link_text = text
    else:
        link_text = mail

    return mark_safe('<a href="{:s}{:s}">{:s}</a>'.format(obfuscate_string('mailto:'), mail, link_text))



# Generated by Django 3.1.2 on 2021-03-26 09:37
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ForeignKeyRelatedModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'single_signal',
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='testapp.mysinglesignalmodel'),
                ),
            ],
        ),
        migrations.CreateModel(
            name='CommonInfoBasedModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'created_at',
                    models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Erstellt am'),
                ),
                (
                    'lastmodified_at',
                    models.DateTimeField(
                        db_index=True, default=django.utils.timezone.now, verbose_name='Zuletzt geändert am'
                    ),
                ),
                ('value', models.PositiveIntegerField(default=0)),
                (
                    'created_by',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name='testapp_commoninfobasedmodel_created',
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='Erstellt von',
                    ),
                ),
                (
                    'lastmodified_by',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name='testapp_commoninfobasedmodel_lastmodified',
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='Zuletzt geändert von',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.test import TestCase

from ai_django_core.utils import clear_cache


class CacheUtilTest(TestCase):
    def test_clear_cache_pseudo(self):
        # This test just executes the method to ensure the api from django is still as we expect it to be
        clear_cache()



import datetime

from django.test import TestCase
from freezegun import freeze_time

from testapp.models import CommonInfoBasedModel


class CommonInfoTest(TestCase):

    @freeze_time('2022-06-26 10:00')
    def test_save_update_fields_common_fields_set(self):
        with freeze_time('2020-09-19'):
            obj = CommonInfoBasedModel.objects.create(value=1)
        obj.value = 2
        obj.save(update_fields=('value',))

        self.assertEqual(obj.value, 2)
        self.assertEqual(obj.lastmodified_at, datetime.datetime(2022, 6, 26, 10))

    @freeze_time('2022-06-26 10:00')
    def test_save_common_fields_set_without_update_fields(self):
        with freeze_time('2020-09-19'):
            obj = CommonInfoBasedModel.objects.create(value=1)
        obj.value = 2
        obj.save()

        self.assertEqual(obj.value, 2)
        self.assertEqual(obj.lastmodified_at, datetime.datetime(2022, 6, 26, 10))
